import 'dart:math';
import 'package:jaguar_reflect/jaguar_reflect.dart';
import 'package:jaguar/jaguar.dart';

final _random = new Random.secure();

class WrapRandomNumberGen extends RouteWrapper<RandomNumberGen> {
  RandomNumberGen createInterceptor() => new RandomNumberGen();
}

class RandomNumberGen extends Interceptor {
  int pre() => _random.nextInt(1000);
}

class WrapDoubler extends RouteWrapper<Doubler> {
  const WrapDoubler();

  Doubler createInterceptor() => new Doubler();
}

class Doubler extends Interceptor {
  int pre(@Input(RandomNumberGen) int number) => number * 2;
}

@Api(path: '/api')
class ExampleApi {
  final WrapRandomNumberGen randomNumberGen = new WrapRandomNumberGen();

  final WrapDoubler doubler = new WrapDoubler();

  @Get(path: '/hello')
  @Wrap(const [#randomNumberGen, #doubler])
  Map sayHello(
          @Input(RandomNumberGen) int random, @Input(Doubler) int doubled) =>
      {'random': random, 'doubled': doubled};
}

main() async {
  final api = new ExampleApi();

  final server = new Jaguar();
  server.addApi(reflectJaguar(api));
  await server.serve();
}
