part of jaguar_reflect.base;

class ReflectedInterceptor {
  final Wrapper wrapper;

  final j.Interceptor inter;

  final InstanceMirror interMirror;

  final _Pre pre;

  final _Post post;

  ReflectedInterceptor(this.wrapper, this.inter, this.interMirror)
      : pre = wrapper.pre,
        post = wrapper.post;

  Future<dynamic> invokePre(j.Context ctx, j.Response response) =>
      pre.invoke(this, ctx, response);

  Future<j.Response> invokePost(j.Context ctx, j.Response response) =>
      post.invoke(this, ctx, response);
}

class _Pre {
  final List<Inject> injects;

  _Pre(this.injects);

  factory _Pre.build(MethodMirror m) {
    final List<Inject> injects = _parseReqParamsForPost(m);

    return new _Pre(injects);
  }

  Future<dynamic> invoke(
      ReflectedInterceptor inter, j.Context ctx, j.Response response) async {
    // Prepare parameters to call Pre
    final List reqParam = [];

    for (Inject inj in injects) {
      reqParam.add(_makeParam(inj, ctx, response));
    }

    // Call Pre
    dynamic result = inter.interMirror.invoke(#pre, reqParam).reflectee;
    if (result is Future) {
      result = await result;
    }

    return result;
  }
}

class _Post {
  final List<Inject> injects;

  _Post(this.injects);

  factory _Post.build(MethodMirror m) {
    final List<Inject> injects = _parseReqParamsForPost(m);

    return new _Post(injects);
  }

  Future<j.Response> invoke(
      ReflectedInterceptor inter, j.Context ctx, j.Response response) async {
    // Prepare parameters to call Post
    final List reqParam = [];

    for (Inject inj in inter.wrapper.post.injects) {
      reqParam.add(_makeParam(inj, ctx, response));
    }

    dynamic resp = inter.interMirror.invoke(#post, reqParam).reflectee;
    if (resp is Future) {
      resp = await resp;
    }
    if (resp is! j.Response) return null;

    return resp;
  }
}
